==> picolibc_1.8.9.orig.tar.gz: picolibc-1.8.9/newlib/libc/uchar/c32rtomb.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2024 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "uchar-local.h"

size_t
c32rtomb (char *s, char32_t c32, mbstate_t *ps)
{
    static mbstate_t local_state;

    if (ps == NULL)
        ps = &local_state;

    if (!char32_is_valid(c32)) {
        errno = EILSEQ;
        return (size_t) -1;
    }
#if __SIZEOF_WCHAR_T__ == 2
    if (char32_needs_surrogates(c32)) {
        const wchar_t wc[2] = {
            [0] = ((c32 - 0x10000) >> 10) + HIGH_SURROGATE_FIRST,
            [1] = (c32 & 0x3ff) + LOW_SURROGATE_FIRST,
        };
        const wchar_t *wcp = wc;
        return wcsnrtombs(s, &wcp, 2, SIZE_MAX, ps);
    }
#endif
    return wcrtomb(s, (wchar_t) c32, ps);
}

==> picolibc_1.8.10.orig.tar.gz: picolibc-1.8.10/newlib/libc/uchar/c32rtomb.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2024 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "uchar-local.h"

size_t
c32rtomb (char *s, char32_t c32, mbstate_t *ps)
{
    static mbstate_t local_state;

    if (ps == NULL)
        ps = &local_state;

    if (!char32_is_valid(c32)) {
        errno = EILSEQ;
        return (size_t) -1;
    }

#if __SIZEOF_WCHAR_T__ == 2
    if (char32_needs_surrogates(c32) && s != NULL) {
        const wchar_t wc[2] = {
            [0] = ((c32 - 0x10000) >> 10) + HIGH_SURROGATE_FIRST,
            [1] = (c32 & 0x3ff) + LOW_SURROGATE_FIRST,
        };
        const wchar_t *wcp = wc;
        return wcsnrtombs(s, &wcp, 2, SIZE_MAX, ps);
    }
#endif
    return wcrtomb(s, (wchar_t) c32, ps);
}
